import hashlib
import hmac


from functools import wraps
from flask import request, make_response
from flask import current_app
from time import time


def verify_signature(request, timestamp, signature, signing_secret):
    # Verify the request signature of the request sent from Slack
    # Generate a new hash using the app's signing secret and request data

    # Compare the generated hash and incoming request signature
    # Python 2.7.6 doesn't support compare_digest
    # It's recommended to use Python 2.7.7+
    # noqa See https://docs.python.org/2/whatsnew/2.7.html#pep-466-network-security-enhancements-for-python-2-7

    if timestamp is None or signature is None:
        return False

    if abs(time() - int(timestamp)) > 60 * 5:
        return False

    req = str.encode('v0:' + str(timestamp) + ':') + request.get_data()
    request_hash = 'v0=' + hmac.new(
        str.encode(signing_secret),
        req, hashlib.sha256
    ).hexdigest()

    return hmac.compare_digest(request_hash, signature)


def slack_sig_auth(f):
    @wraps(f)
    def validate_slack_signature(*args, **kwargs):
        slack_signing_secret = current_app.config['SLACK_SIGNING_SECRET'] or os.environ['SLACK_SIGNING_SECRET']
        req_timestamp = request.headers.get('X-Slack-Request-Timestamp')
        req_signature = request.headers.get('X-Slack-Signature')

        if not verify_signature(request, req_timestamp, req_signature, slack_signing_secret):
            return make_response("Unauthorized", 403)
        return f(*args, **kwargs)
    return validate_slack_signature



from setuptools import setup
import codecs
import os
import re

here = os.path.abspath(os.path.dirname(__file__))

# Read the version number from a source file.
# Why read it, and not import?
# see https://groups.google.com/d/topic/pypa-dev/0PkjVpcxTzQ/discussion
def find_version(*file_paths):
    # Open in Latin-1 so that we avoid encoding errors.
    # Use codecs.open for Python 2 compatibility
    with codecs.open(os.path.join(here, *file_paths), 'r', 'latin1') as f:
        version_file = f.read()

    # The version line must have the form
    # __version__ = 'ver'
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

with open('README.rst', 'r') as fh:
    long_description = fh.read()

setup(name='flask_slacksigauth',
      version=find_version('flask_slacksigauth', 'version.py'),
      description='Python Flask adapter for verifying Slack signatures',
      long_description=long_description,
      long_description_content_type='text/x-rst',
      url='https://github.com/eaescob/flask-slacksigauth',
      author='Emilio Escobar',
      author_email='eescobar@gmail.com',
      license='GNU',
      packages=['flask_slacksigauth'],
      install_requires=[
        'flask',
        'pyee',
        'requests',
        'six',
      ],
      zip_safe=False)
